UnitScale <- function(x){
  z <- max(branching.times(x))
  x$edge.length <- x$edge.length/z
  return(x)
}

GetOverlap <- function(x, burnin){
  library(coda)
  topnum <- nrow(x) - burnin
  z <- vector()
  for(i in 1:topnum){
    j <- i + burnin
    z[i] <- ((x[j, 2] + x[j, 3]) / 2) -
      ((x[j, 4] + x[j, 5]) / 2)
  }
  z <- HPDinterval(as.mcmc(z), prob=0.95)
  return(z)
}

# this chunk will simulate the trees with state 1 (of 0 and 1)
# having a speciation rate twice that of state 0.

library(diversitree)
pars <- c(3, 6, 1, 1, .5, .5)
trees <- list()
for(i in 1:100){
  foo <- NULL
  while(is.null(foo)){
    foo <- tree.bisse(pars, max.taxa=200, x0=0)
  }
  print(i)
  trees[[i]] <- UnitScale(foo)
}

# this chunk will simulate chromosome number for all the tips
# with no relationship to binary state
for(i in 1:100){
  trees[[i]][[11]] <- sim.character(trees[[i]], model="meristic", x0=10, pars=c(20, 2, 2))
  names(trees[[i]])[[11]] <- "chrome.num"
  names(trees[[i]])[[4]] <- "bi.state"
}

# now we need to evaluate these datasets with and without accounting for
# SSE

library(chromePlus)
x.w <- x.wo <- list()
for(i in 1:100){
  print(paste("working on tree", i))
  cdat <- data.frame(trees[[i]]$tip.label,
                     as.numeric(trees[[i]]$chrome.num),
                     as.numeric(trees[[i]]$bi.state))
  colnames(cdat) <- c("taxa", "haploid", "bin")
  z <- range(cdat[,2])
  z[1] <- z[1] - 1
  z[2] <- z[2] + 1
  cmat <- datatoMatrix(cdat, range=z, hyper=T)
  lk.mk <- make.musse(trees[[i]], states=cmat, k=ncol(cmat), 
                    strict=F, control=list(method="ode"))
  wo.sse <-constrainMuSSE(data=cmat, 
                          lik=lk.mk, 
                          hidden=T, polyploidy=F,
                          verbose=F, 
                          s.lambda = T, s.mu = T,
                          constrain=list(drop.demi=T, drop.poly=T))
  w.sse <-constrainMuSSE(data=cmat, 
                         lik=lk.mk, 
                         hidden=T, polyploidy=F,
                         verbose=F, 
                         s.lambda = F, s.mu = F,
                         constrain=list(drop.demi=T, drop.poly=T))
  x.w[[i]] <- diversitree::mcmc(w.sse, print.every = 25,
                           lower=rep(0,10),
                           upper=rep(20,10),
                           x.init=runif(min=0,max=5, 10), 
                           w=1, nsteps=1000)
  x.wo[[i]] <- diversitree::mcmc(wo.sse, print.every = 25,
                           lower=rep(0,8),
                           upper=rep(20,8),
                           x.init=runif(min=0,max=5, 8), 
                           w=1, nsteps=1000)
}


results <- as.data.frame(matrix(,100,11))
colnames(results) <- c("tree", "deltar.wo", "deltar.w", 
                       "hpd.95.wo.l", "hpd.95.wo.h", 
                       "hpd.95.w.l", "hpd.95.w.h",
                       "trans.wo.12", "trans.wo.21", 
                       "trans.w.12", "trans.w.21")
# x mean rate in 1 wo
# y mean rate in 2 wo
# r mean rate in 1 w
# s mean rate in 2 w
burnin <- 750
for(i in 1:100){
  x <- mean(c(x.wo[[i]]$asc1[burnin:1000], x.wo[[i]]$desc1[burnin:1000]))
  y <- mean(c(x.wo[[i]]$asc2[burnin:1000], x.wo[[i]]$desc2[burnin:1000]))
  r <- mean(c(x.w[[i]]$asc1[burnin:1000], x.w[[i]]$desc1[burnin:1000]))
  s <- mean(c(x.w[[i]]$asc2[burnin:1000], x.w[[i]]$desc2[burnin:1000]))
  results[i,1] <- i
  results[i,2] <- x-y
  results[i,3] <- r-s
  results[i,4:5] <- as.numeric(GetOverlap(x.wo[[i]], burnin=500))
  results[i,6:7] <- as.numeric(GetOverlap(x.w[[i]], burnin=500))
  results[i,8] <- mean(x.wo[[i]]$tran12[burnin:1000])
  results[i,9] <- mean(x.wo[[i]]$tran21[burnin:1000])
  results[i,10] <- mean(x.w[[i]]$tran12[burnin:1000])
  results[i,11] <- mean(x.w[[i]]$tran21[burnin:1000])
}


# check to see if means are different
plot(results$mean.wo, col="red", pch=16,cex=.6,ylim=c(-5,5))
points(results$mean.w, col="blue", pch=16,cex=.6)
boxplot(results[,2:3])
boxplot(results[8:11], names=c("a","b","c","d"))

write.csv(results,file="SSE.sim.csv")

sig.w <- sig.wo <- as.logical(rep("TRUE", 100))
for(i in 1:100){
  if(min(results[i, 4:5]) < 0 & max(results[i, 4:5]) > 0) sig.wo[i] <- FALSE
  if(min(results[i, 6:7]) < 0 & max(results[i, 6:7]) > 0) sig.w[i] <- FALSE
}

